# --------------------
package Maps::Controller::Pages;
# --------------------
use Mojo::Base 'Maps::Controller';
use Mojo::JWT;
use Const::Fast;
use DDP;

our $VERSION = '0.2';

const my $EMPTY => q{};
const my $OK => 200;
const my $UNAUTHORIZED => 401;

sub options {
	my $c = shift;
	return $c
	  ->addCORSHeaders
		->addAuthHeaders
		->respond_to(any => {
			data => $EMPTY,
			status => $OK,
			});
}

sub login {
  my $c = shift;
  return $c->app->is_user_authenticated()
    ? $c->redirect_to('/maps')
    : $c->render('pages/login');
};

sub authenticate {
  my $c = shift;
  my $user = $c->req->param('user');
  $c->helpers->authenticate($user, $c->req->param('pass')) if $user;
  if ($c->is_user_authenticated()) {
    $c->flash(obs => 'Angemeldet');
    $user = $c->current_user->{name};
    $c->app->log->debug("Logged in: $user");
    }
  else {
    $c->flash(obs => 'Anmeldung fehlgeschlagen');
    $c->app->log->debug('Not logged in');
    }
  return $c->redirect_to("/");
};

sub logout {
  my $c = shift;
  my $user = $c->current_user;
  $c->app->log->debug("Logged out: $user->{name}") if $user;
  $c->helpers->logout;
  $c->flash(ok => 'Abgemeldet');
  return $c->redirect_to('/login');
};

sub token {
  my $c = shift;
  return $c->authorize;
};

sub validate {
	my $c = shift;
	$c->addCORSHeaders;
	my $token = $c->req->headers->header('x-annotator-auth-token');
	my $claims = $c->validateToken($token);
  # $c->app->log->debug($token, p $claims);
  return $c->reject unless defined $claims;
	return $c->stash('user' => $claims->{userId});
};

sub reject {
	my $c = shift;
	$c->render(text => 'Unauthorized', status => $UNAUTHORIZED);
	return;
};

__PACKAGE__
