# --------------------
package Maps::Controller::Top;
# --------------------
use Mojo::Base 'Maps::Controller';
use Mojo::JSON 'encode_json';
use Mojo::Template;
use Mojo::URL;

use Const::Fast;
use List::Util qw(max min);
use DDP;

our $VERSION = '0.2';

const my $EMPTY => q{};
const my $SPACE => q{ };
const my $NO_CONTENT => 204;

#	DBI TraceLevel 	Description
	#	0   Trace disabled.
	#	1   Trace top-level DBI method calls returning with results or errors.
	#	2   As above, adding tracing of top-level method entry with parameters.
	#	3   As above, adding some high-level information from the driver
	#	    and some internal information from the DBI.
	#	4   As above, adding more detailed information from the driver.
	#	    This is the first level to trace all the rows being fetched.
	#	5-15 As above but with more and more internal information

#	$db->dbh->trace( 1, 'dbi_trace.log' );

#	$ENV{DBI_TRACE} = "1=dbi_trace.log";

*_f = \&CORE::sprintf;

my $JSONFIELDS = [qw(
  json_top json_refs json_maps json_guides json_origin json_bib
  )];

my $_allFields = [qw(
  id
  id_map id_group id_top fmid qid fn name nr legend desc type shape bbox tags comment
  state user created
  )];

my $_updateFields = [ @$_allFields[1..15] ];
my $_countAll = @$_allFields;
my $_countUpdate = @$_updateFields;
my $_insertFieldnames = join (q{, }, $_allFields);
my $_insertPlaceholders  = join(q{, }, ('?') x ($_countAll - 1));
my $_updatePlaceholders = join (q{, }, map { "$_ = ?" } $_countUpdate);

sub _renderCached {
  my ($c, $result) = @_;
  $c->res->headers->cache_control('max-age=3600, must-revalidate');
  return $c->render(json => $result);
}

# add CSS, shapes, permissions and textfield to the db result
sub _jsonCreate {
  my ($c, $json) = @_;
  return $json;
}

sub all {
  my ($c, $format) = @_;
  my $id = $c->stash('id');
  my $res = $c->db->query('SELECT * FROM top ORDER BY toponym, date_from')
    ->expand(json => $JSONFIELDS)
    ->hashes;
  $c->DEBUG("TOP: all - result count: " . $res->size);
  return $c->render('topall', res => $res);
}

# gazetteer: collect all the data for one toponym item
sub item {
  my ($c, $format) = @_;
  my $id = $c->stash('id');
  my $res = $c->db->query('SELECT * FROM top_all WHERE id = ?', $id)
    ->expand(json => $JSONFIELDS)
    ->hash;
  $c->DEBUG("TOP: item - result: $id");
  return $res if $format;
  $c->render('topitem', res => $res);
}

sub json {
  my $c = shift;
  my $json = $c->item('json');
  $c->DEBUG("TOP: rendering as json");
  $c->render(json => $json);
}

__PACKAGE__

