# --------------------
package Mojolicious::Plugin::Token;
# --------------------
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::JWT;

use Authen::Simple::Kerberos;
use Time::Piece;
use Const::Fast;

const my $ALGORITHM => 'RS512';
const my $ALL => '*';
const my $EMPTY => q{};
const my $TRUE => 'true';
const my $TTL => 86400;
const my $UNAUTHORIZED => 401;

our $VERSION = '0.1';

sub register {
  my ($self, $app) = @_;
  my $config = $app->config;

  $app->helper(addCORSHeaders => sub {
    my $c = shift;
    my $origin = $c->req->headers->origin;
    $c->res->headers->access_control_allow_origin($origin || $ALL);
    $c->res->headers->header('Access-Control-Allow-Credentials' => $TRUE);
    return $c;
  	});

  $app->helper(addAuthHeaders => sub {
    my $c = shift;
    $c->res->headers->header('Access-Control-Allow-Methods' => 'GET, OPTIONS, POST, DELETE, PUT');
    $c->res->headers->header('Access-Control-Allow-Headers' => 'content-type,x-annotator-auth-token,x-csrftoken');
    return $c;
  	});

  $app->helper(authorize => sub {
  	my $c = shift;
		my $user = $c->req->param('user') || $EMPTY;
		my $pass = $c->req->param('password') || $EMPTY;
		$c->addCORSHeaders;
		return $c->isAuthenticated($user, $pass)
		  ? $c->render(text => $c->issueToken($user))
		  : $c->render(text => 'Unauthorized', status => $UNAUTHORIZED);
		});

  $app->helper(isAuthenticated => sub {
    my ($c, $user, $pass) = @_;
    return unless $user;
    my $domain = Authen::Simple::Kerberos->new(
  	  realm => $config->{kerberos_realm},
  	  );
    my $ok = (
    		$domain->authenticate($user, $pass)
        or exists $config->{users}->{$user} && $config->{users}->{$user} eq $pass
        );
    $c->DEBUG("Token::isAuthenticated: $user/$pass:$ok");
    return $ok;
  	});

  $app->helper(issueToken => sub {
    my ($c, $user) = @_;
    my $claims = {
			userId => $user,
			consumerKey => $config->{name},
			issuedAt => localtime->ymd(),
			ttl => $TTL,
			};
		return Mojo::JWT->new(
			claims => $claims,
			secret => $config->{private},
			algorithm => $ALGORITHM,
			)->encode;
  	});

  $app->helper(validateToken => sub {
    my ($c, $token) = @_;
    return eval {
			Mojo::JWT->new( public => $config->{public}, )
				->decode($token // $EMPTY);
			};
  	});

  }

__PACKAGE__
