#! perl
use Mojo::Base 'strict';
use Mojo::File 'path';
use Mojo::Collection 'c';

no warnings 'uninitialized';

use Data::Printer {
  output => 'stdout',
	quote_keys => 1,
	# filters => { SCALAR => [ sub { return "q{${$_[0]}}" },  ], },
	hash_separator => ' => ',
	align_hash => 0,
};

# raw data
my $file = path("atlante-napoli-palazzi-all.txt")->slurp;
my $lines = c(split /\n/, $file);

my $MAIN = {};
my $OUT = {};

my $types = {};
my $lemmas = c;
my $crefs = c;

sub getLemma {
  my $line = shift;
  my ($lemma) = ($line =~ /^([^\(]+) /);
  return $lemma;
}

sub getPart2 {
  my $line = shift;
  my ($part2) = ($line =~ /\)(.*?) [\d=][:>]/);
  return $part2;
}

sub getLongName {
  my ($nr, $type, $lemma, $rest) = @_;
  $rest //= '';
  return "[$nr] type: $type (Atlante $rest)";
}

sub getPages {
  my ($line) = @_;
  my ($pages) = ($line =~ /(\d+:[\d,nd]+)/);
  return $pages;
}

sub getType {
  my $line = shift;
  $line =~ s/^(ex|due|tre|quattro|coppia di) //; # delete numbers
  $line =~ s/^(\w+)\-\w+/$1/; # delete fondaco in palazzo-fondaco
  $line =~ s/^palazz\w+/palazzo/;
  $line =~ s/^edi\w+/edificio/;
  $line =~ s/^cas\w+/casa/;
  $line =~ s/^vill\w+/villa/;
  $line = "other $line" unless $line =~ /^(cas|palaz|edif|vill|mass)/;
  my @names = split /\s+/, $line;
  return $names[0];
}

$lines->each( sub {
  # line
  my $nr = $_[1];
  my ($paren) = ($_ =~ /\(([^\)]+)\)/);

  return unless $paren;
  my $type = getType($paren);
  my $pages = getPages($_);
  my $lemma = getLemma($_);
  my $part2 = getPart2($_);
  my $name = ucfirst($paren) . ' ' . $lemma;
  $name .= $part2 if $part2;
  $name =~ s/'/\\'/g; # no way to change hash key quoting in DDP?
  my $set = $OUT->{$name} //= [];

  # store
  $MAIN->{$name}++;
  unshift @$set, getLongName($nr, $type, $lemma, $pages);

  # crossref
  my ($cref) = ($_ =~ /=> (.+)\r/);
  return unless $cref;
  ($paren) = ($cref =~ /\(([^\)]+)\)/);

  $pages ||= getPages($cref) || '';
  $cref =~ s/ \d:.*//;
  my $name1 = $paren ? ucfirst($paren) . ' ' . getLemma($cref) : ucfirst($cref);
  my $set1 = $OUT->{$name1} //= [];

  # store
  $MAIN->{$name1}++;


  push @$set, "[$nr] --> $name1";
  push @$set1, "[$nr] <-- $name (Atlante $pages)";
});

::p $OUT;
say STDERR scalar (keys %$MAIN);

# say ($_, " : ", $MAIN->{$_}) for sort keys %$MAIN;
# $crefs->sort->each(sub {say $_ });


