#! perl
use Mojo::Base 'strict';
use Mojo::File 'path';
use Mojo::Collection 'c';

no warnings 'uninitialized';

use Data::Printer {
  output => 'stdout',
	quote_keys => 1,
	# filters => { SCALAR => [ sub { return "q{${$_[0]}}" },  ], },
	hash_separator => ' => ',
	align_hash => 0,
};

# raw data
my $file = path("chiese_1.txt")->slurp;
my $lines = c(split /\n/, $file);

my $CHIESE = {};
my $MAIN = {};

$lines->each(sub {
	my ($name1, $pre1, $vol, $types1, $pages1, $arrow, $pre2, $name2, $types2, $pages2) = split( /\|/, $_);
    # filter out and save sequence of page numbers
    my $pages = $pages1 || $pages2;
	# reference?
	my $ref = $arrow =~ /=>/;
	# types separated by 'e', 'ed'
	my @types1 = split / ed? /, $types1;
	my @types2 = split / ed? /, $types2;
	# data
    if ($ref) {
	  for my $t (@types2) {
	  	# store referenced targets as main entries for a church
	    my $item = $CHIESE->{$t}->{$name2} ||= {};
	    # prefix = S., San, Santi, SS.
	    $item->{prefix}->{$pre2}++ if $pre2;
	    $item->{$vol} = $pages if ($vol && $pages);
	    my $r = $item->{ref} ||= {};
	    # attach references
	    for my $tt (@types1) {
	    		push @{$r->{$tt}}, { $name1 => $pre1 };
	    		}
	    	# add item to main entry list
	    $MAIN->{"$t - $name2"}++;
	    }
    	}
    	else {
	  for my $t (@types1) {
	  	# store items without references as main entries for a church
	    my $item = $CHIESE->{$t}->{$name1} ||= {};
	    # prefix = S., San, Santi, SS.
	    $item->{prefix}->{$pre1}++ if $pre1;
	    $item->{$vol} = $pages if ($vol && $pages);

	    	# add item to main entry list
	    $MAIN->{"$t - $name1"}++;
	    }
	  }

});

::p $CHIESE;
say STDERR scalar (keys %$MAIN);
# say ($_, " : ", $MAIN->{$_}) for sort keys %$MAIN;

