#! perl
use Mojo::Base 'strict';
use Mojo::File 'path';
use Mojo::Collection 'c';
use JSON::XS;
use Text::Levenshtein::XS qw(distance);
use Text::JaroWinkler qw( strcmp95 );

no warnings 'uninitialized';

use Data::Printer {
  output => 'stdout',
	quote_keys => 1,
	# filters => { SCALAR => [ sub { return "q{${$_[0]}}" },  ], },
	hash_separator => ' => ',
	align_hash => 0,
};

my $CHIESE = {};
my $MAIN = {};

# raw data, collected from all 'Atlante di Napoli' indexes
my $file = path("atlante/chiese_1.txt")->slurp;
my $lines = c(split /\n/, $file);

sub max { $_[$_[0] < $_[1]] }
sub maxlen { max(length($_[0]), length($_[1])) }

# construct hash of all churches by type, then name
sub churchHash {
  my ($self) = @_;
  $lines->each(sub {
  	my ($name1, $pre1, $vol, $types1, $pages1, $arrow, $pre2, $name2, $types2, $pages2) = split( /\|/, $_);
  	# filter out and save sequence of page numbers
  	my $pages = $pages1 || $pages2;
  	# reference?
  	my $ref = $arrow =~ /=>/;
  	# types separated by 'e', 'ed'
  	my @types1 = split / ed? /, $types1;
  	my @types2 = split / ed? /, $types2;
  	# data
  	if ($ref) {
  	  for my $t (@types2) {
  		# store referenced targets as main entries for a church
  		my $item = $CHIESE->{$t}->{$name2} ||= {};
  		# prefix = S., San, Santi, SS.
  		$item->{prefix}->{$pre2}++ if $pre2;
  		$item->{$vol} = $pages if ($vol && $pages);
  		my $r = $item->{ref} ||= {};
  		# attach references
  		for my $tt (@types1) {
  		  push @{$r->{$tt}}, { $name1 => $pre1 };
  		  }
  		# add item to main entry list
  		  $MAIN->{$name2}++;
  	    }
  	  }
  	else {
  	  for my $t (@types1) {
  		# store items without references as main entries for a church
  		my $item = $CHIESE->{$t}->{$name1} ||= {};
  		# prefix = S., San, Santi, SS.
  		$item->{prefix}->{$pre1}++ if $pre1;
  		$item->{$vol} = $pages if ($vol && $pages);

  		# add item to main entry list
  		$MAIN->{$name1}++;
  		}
  	  }
  });
}

# ::p $CHIESE;

# say STDERR scalar (keys %$MAIN);

churchHash();
my $churches = c(sort keys %$MAIN);

# say ($_, " : ", $MAIN->{$_}) for sort keys %$MAIN;
my $j = JSON::XS->new->utf8(0);
my $fmdata = bless $j->decode(path('fm/napoli-fm.json')->slurp), 'Mojo::Collection';
my $data = $fmdata->map( sub { $_->{name} })->sort;

$churches->each( sub {
  my $church = $_;
  my ($max, $dist);
  $data->each( sub {
    my $name = $_;
    return if $name =~ /^Palazzo|Museo|Cimitero|Porta|Fontana|Castel|Piazza/;
    $name =~ s/^(Chiesa d\S+ |SS?\. )//;
	  my $d = strcmp95($name, $church, maxlen($name, $church));
    if ($d >= $dist) {
      $dist = $d;
      $max = $name;
      };
    });
    say $dist > 0.9 ? "$church = $max ($dist)" : "-- $church";
  });
## ::p $data;

