#! perl
use Mojo::Base 'strict';
use Mojo::File 'path';
use Mojo::Collection 'c';
use Text::Levenshtein::XS qw(distance);
use Text::WagnerFischer qw(distance);
use Text::JaroWinkler qw(strcmp95);

no warnings 'uninitialized';

use Data::Printer {
  output => 'stdout',
	quote_keys => 1,
	# filters => { SCALAR => [ sub { return "q{${$_[0]}}" },  ], },
	hash_separator => ' => ',
	align_hash => 0,
};

sub max { $_[$_[0] < $_[1]] }
sub maxlen { max(length($_[0]), length($_[1])) }

my $RE = qr/((basilica|chiesa) (dei |di |del |dell')|s\. |san |sant'|ss\. |santa |santi )/;

# raw data
my $file = path("./napoli-fm.tab")->slurp;
my $lines = c(split /[\r\n]+/, $file);

my $FM = {};

$lines->each(sub {
  my $line = $_;
  my ($fmid, $name, $alias) = split /\|/, $line;
  $alias = lc($alias);
  $name = lc($name);
  $name =~ s/$RE//g;
  $alias =~ s/$RE//g;
  $FM->{$name} = $fmid;
  $FM->{$alias} = $fmid if $alias;
  });


# raw data
$file = path("../wikidata/napoli-2019.txt")->slurp;
$lines = c(split /[\r\n]+/, $file);

# ?item|?itemLabel|?itemAltLabel|?itemDescription|?geoloc|?gnd|?viaf|?typ|?img|?link
my $WD = {};

$lines->each(sub {
  my $line = $_;
  my ($id, $name, $alias, @data) = split /\|/, $line;
  $alias = lc($alias);
  $name = lc($name);
  $id =~ s/Q//;
  $name =~ s/$RE//g;
  $alias =~ s/$RE//g;
  $WD->{$name} = $id;
  $WD->{$alias} = $id if $alias;
#  say $name;
  });


# say STDERR "FM-Daten:", scalar (keys %$FM);
# ::p $FM;

my $match = c;
my $near = c;
my $miss = c;

say STDERR "WD-Daten:", scalar (keys %$WD);
# ::p $WD;
my ($y, $n);
c(keys %$FM)->each( sub {
  my $fmkey = $_;
  if ($WD->{$fmkey}) {
    say "= $fmkey";
    push @$match, $fmkey;
    return;
    }
  my $max = 0;
  my $dist = 0;
  c(keys %$WD)->each( sub {
    my $wdkey = $_;
 	  my $diff = strcmp95($wdkey, $fmkey, maxlen($wdkey, $fmkey));
    if ($diff >= $dist) {
      $dist = $diff;
      $max = $wdkey;
      };
    });
  if ($dist > 0.96) {
    push @$near, "$fmkey ~ $max ($dist)";
  } else {
    push @$miss, "$fmkey / $max ($dist)";
  }
  say $match->sort->join("\n");
  say "\nNear";
  say $near->sort->join("\n");
  say "\nMiss";
  say $miss->sort->join("\n");
  });
